try:
    from django.template import Library
    from django.template.defaultfilters import stringfilter
except:
    raise Exception('Django is not installed.')

from twitter_text import TwitterText

register = Library()

@register.filter(name = 'twitter_text')
@stringfilter
def twitter_text(text, search_query = False):
    """
    Parses a text string through the TwitterText auto_link method and if search_query is passed, through the hit_highlight method.
    """
    tt = TwitterText(text)
    if search_query:
        tt.highlighter.hit_highlight(search_query)
    tt.autolink.auto_link()
    return tt.text
twitter_text.is_safe = True


# encoding=utf-8

from twitter_text.regex import REGEXEN
from twitter_text.unicode import force_unicode

class Extractor(object):
    """
    A module for including Tweet parsing in a class. This module provides function for the extraction and processing
    of usernames, lists, URLs and hashtags.
    """
    
    def __init__(self, text):
        self.text = force_unicode(text)
    
    def extract_mentioned_screen_names(self, transform = False):
        """
        Extracts a list of all usernames mentioned in the Tweet text. If the
        text contains no username mentions an empty list will be returned.
        
        If a transform is given, then it will be called with each username.
        """
        screen_names_only = []
        matches = self.extract_mentioned_screen_names_with_indices()
        for screen_name in matches:
            if transform:
                screen_name['screen_name'] = transform(screen_name['screen_name'])
            screen_names_only.append(screen_name['screen_name'])
            del(screen_name)
        del(matches)
        del(transform)
        return screen_names_only

    def extract_mentioned_screen_names_with_indices(self, transform = False):
        """
        Extracts a list of all usersnames mentioned in the Tweet text
        along with the indices for where the mention ocurred.  If the
        text contains no username mentions, an empty list will be returned.
        
        If a transform is given, then it will be called with each username, the start
        index, and the end index in the text.
        """
        possible_screen_names = []
        matches = REGEXEN['extract_mentions'].finditer(self.text)
        for match in matches:
            start = match.start() + len(match.group(1))
            end = start + len(match.group(2)) + len(match.group(3))
            if transform:
                possible_screen_name = transform(match.group(3), start, end)
            else:
                possible_screen_name = {
                    'screen_name': match.group(3),
                    'indices': (start, end)
                }
            possible_screen_names.append(possible_screen_name)
            del(possible_screen_name)
        del(matches)
        del(transform)
        return possible_screen_names
        
    def extract_reply_screen_name(self, transform = False):
        """
        Extracts the first username replied to in the Tweet text. If the
        text does not contain a reply None will be returned.
        
        If a transform is given then it will be called with the username replied to (if any)
        """
        possible_screen_name = REGEXEN['extract_reply'].match(self.text)
        if possible_screen_name is not None:
            possible_screen_name = possible_screen_name.group(1)
        if transform:
            possible_screen_name = transform(possible_screen_name)
        del(transform)
        return possible_screen_name
        
    def extract_urls(self, transform = False):
        """
        Extracts a list of all URLs included in the Tweet text. If the
        text contains no URLs an empty list will be returned.
        
        If a transform is given then it will be called for each URL.
        """
        urls_only = []
        matches = self.extract_urls_with_indices()
        for url in matches:
            if transform:
                url['url'] = transform(url['url'])
            urls_only.append(url['url'])
            del(url)
        del(matches)
        del(transform)
        return urls_only
        
    def extract_urls_with_indices(self, transform = False):
        """
        Extracts a list of all URLs included in the Tweet text along
        with the indices. If the text contains no URLs an empty list
        will be returned.
        
        If a transform is given then it will be called for each URL.
        """
        urls = []
        matches = REGEXEN['valid_url'].finditer(self.text)
        for match in matches:
            start = match.start() + len(match.group(1))
            end = start + len(match.group(2))
            if transform:
                url = transform(match.group(2), start, end)
            else:
                url = {
                    'url': match.group(2),
                    'indices': (start, end)
                }
            urls.append(url)
            del(url)
        del(matches)
        del(transform)
        return urls
        
    def extract_hashtags(self, transform = False):
        """
        Extracts a list of all hashtags included in the Tweet text. If the
        text contains no hashtags an empty list will be returned.
        The list returned will not include the leading # character.
        
        If a transform is given then it will be called for each hashtag.
        """
        hashtags_only = []
        matches = self.extract_hashtags_with_indices()
        for hashtag in matches:
            if transform:
                hashtag['hashtag'] = transform(hashtag['hashtag'])
            hashtags_only.append(hashtag['hashtag'])
            del(hashtag)
        del(matches)
        del(transform)
        
        return hashtags_only
        
    def extract_hashtags_with_indices(self, transform = False):
        """
        Extracts a list of all hashtags included in the Tweet text. If the
        text contains no hashtags an empty list will be returned.
        The list returned will not include the leading # character.
        
        If a transform is given then it will be called for each hashtag.
        """
        tags = []
        matches = REGEXEN['auto_link_hashtags'].finditer(self.text)
        for match in matches:
            start = match.start() + len(match.group(1))
            end = start + len(match.group(2)) + len(match.group(3))
            if transform:
                tag = transform(match.group(3), start, end)
            else:
                tag = {
                    'hashtag': match.group(3),
                    'indices': (start, end)
                }
            tags.append(tag)
            del(tag)
        del(matches)
        del(transform)
        
        return tags


# encoding=utf-8

import re

from twitter_text.regex import REGEXEN
from twitter_text.unicode import force_unicode

class Autolink(object):
    
    WWW_REGEX = re.compile(r'www\.', re.IGNORECASE)

    DEFAULT_URL_CLASS = 'tweet-url'

    DEFAULT_LIST_CLASS = 'list-slug'

    DEFAULT_USERNAME_CLASS = 'username'

    DEFAULT_HASHTAG_CLASS = 'hashtag'

    HTML_ATTR_NO_FOLLOW = ' rel="nofollow"'

    def __init__(self, text, **kwargs):
        self.text = force_unicode(text)
        self.parent = kwargs.get('parent', False)

    def auto_link(self, **kwargs):
        """
        Add <a></a> tags around the usernames, lists, hashtags and URLs in the provided text. The
        <a> tags can be controlled with the following kwargs
    
        url_class::     class to add to all <a> tags
        list_class::    class to add to list <a> tags
        username_class::    class to add to username <a> tags
        hashtag_class::    class to add to hashtag <a> tags
        username_url_base::      the value for href attribute on username links. The @username (minus the @) will be appended at the end of this.
        list_url_base::      the value for href attribute on list links. The @username/list (minus the @) will be appended at the end of this.
        hashtag_url_base::      the value for href attribute on hashtag links. The #hashtag (minus the #) will be appended at the end of this.
        suppress_lists::    disable auto-linking to lists
        suppress_no_follow::   Do not add rel="nofollow" to auto-linked items
        html_attrs::    A dictionary of HTML attributes to add to non-Twitter links
        """
        self.auto_link_urls_custom(**kwargs.get('html_attrs', {}))
        self.auto_link_hashtags(**kwargs)
        self.auto_link_usernames_or_lists(**kwargs)

        if self.parent and hasattr(self.parent, 'text'):
            self.parent.text = self.text
        if self.parent and hasattr(self.parent, 'has_been_linked'):
            self.parent.has_been_linked = True

        return self.text
    
    def auto_link_usernames_or_lists(self, **kwargs):
        """
        Add <a></a> tags around the usernames and lists in the provided text. The
        <a> tags can be controlled with the following kwargs
    
        url_class::     class to add to all <a> tags
        list_class::    class to add to list <a> tags
        username_class::    class to add to username <a> tags
        username_url_base::      the value for href attribute on username links. The @username (minus the @) will be appended at the end of this.
        list_url_base::      the value for href attribute on list links. The @username/list (minus the @) will be appended at the end of this.
        suppress_lists::    disable auto-linking to lists
        suppress_no_follow::   Do not add rel="nofollow" to auto-linked items
        """
        defaults = {
            'url_class': self.DEFAULT_URL_CLASS,
            'list_class': self.DEFAULT_LIST_CLASS,
            'username_class': self.DEFAULT_USERNAME_CLASS,
            'username_url_base': 'http://twitter.com/',
            'list_url_base': 'http://twitter.com/',
        }
        for key, val in defaults.items():
            if not key in kwargs:
                kwargs[key] =   val
        extra_html = '' if kwargs.get('suppress_no_follow', False) else self.HTML_ATTR_NO_FOLLOW
    
        matches = REGEXEN['auto_link_usernames_or_lists'].finditer(self.text)
        for match in matches:
            _link = match.group(0)
            if match.group(4) is not None and not kwargs.get('suppress_lists', False):
                # this link is a list
                _list = u'%s%s' % (match.group(3), match.group(4))
                _link = u'%s%s<a class="%s" href="%s%s"%s>%s</a>' % ( match.group(1), match.group(2), ' '.join( [ kwargs.get('url_class', ''), kwargs.get('list_class', '') ] ), kwargs.get('list_url_base'), _list.lower(), extra_html, _list )
                del(_list)
            else:
                # this is a screen name
                _username = match.group(3)
                _link = u'%s<a class="%s" href="%s%s"%s>%s%s</a>' % ( match.group(1), ' '.join( [ kwargs.get('url_class'), kwargs.get('username_class', '') ] ), kwargs.get('username_url_base', ''), _username, extra_html, match.group(2), _username )
                del(_username)
            self.text = self.text.replace(match.group(0), _link)
            del(_link)

        if self.parent and hasattr(self.parent, 'text'):
            self.parent.text = self.text
        if self.parent and hasattr(self.parent, 'has_been_linked'):
            self.parent.has_been_linked = True

        del(matches)
        del(extra_html)
        del(kwargs)
        del(defaults)
        
        return self.text
    
    def auto_link_hashtags(self, **kwargs):
        """
        Add <a></a> tags around the hashtags in the provided text. The
        <a> tags can be controlled with the following kwargs
    
        url_class::     class to add to all <a> tags
        hashtag_class:: class to add to hashtag <a> tags
        hashtag_url_base::      the value for href attribute. The hashtag text (minus the #) will be appended at the end of this.
        suppress_no_follow::   Do not add rel="nofollow" to auto-linked items
        """
        defaults = {
            'url_class': self.DEFAULT_URL_CLASS,
            'hashtag_class': self.DEFAULT_HASHTAG_CLASS,
            'hashtag_url_base': 'http://twitter.com/search?q=%23',
        }
        for key, val in defaults.items():
            if not key in kwargs:
                kwargs[key] =   val
        extra_html = '' if kwargs.get('suppress_no_follow', False) else self.HTML_ATTR_NO_FOLLOW
    
        matches = REGEXEN['auto_link_hashtags'].finditer(self.text)
        for match in matches:
            _link = u'%s<a href="%s%s" title="#%s" class="%s"%s>%s%s</a>' % ( match.group(1), kwargs.get('hashtag_url_base'), match.group(3), match.group(3), ' '.join( [ kwargs.get('url_class', ''), kwargs.get('hashtag_class', '') ] ), extra_html, match.group(2), match.group(3) )
            self.text = self.text.replace(match.group(0), _link)

        if self.parent and hasattr(self.parent, 'text'):
            self.parent.text = self.text
        if self.parent and hasattr(self.parent, 'has_been_linked'):
            self.parent.has_been_linked = True

        del(matches)
        del(extra_html)
        del(kwargs)
        del(defaults)
        
        return self.text
    
    def auto_link_urls_custom(self, **kwargs):
        """
        Add <a></a> tags around the URLs in the provided text. Any
        elements in kwargs will be converted to HTML attributes
        and place in the <a> tag. Unless kwargs contains suppress_no_follow
        the rel="nofollow" attribute will be added.
        """
        defaults = {}
        if kwargs.get('suppress_no_follow', False):
            del(kwargs['suppress_no_follow'])
        else:
            defaults = {
                'rel': ' '.join( [ kwargs.get('rel', ''), 'nofollow' ] ).strip()
            }
        for key, val in defaults.items():
            if not key in kwargs:
                kwargs[key] =   val

        html_attrs = []
        for k, v in kwargs.items():
            if str(k) == 'class_name': # handle this special case
                k = 'class'
            html_attrs.append( u'%s="%s"' % ( str(k), str(v) ) )
        html_attrs = ' '.join(html_attrs)
        if len(html_attrs):
            html_attrs = ' ' + html_attrs
        
        matches = REGEXEN['valid_url'].finditer(self.text)
        for match in matches:
            full_url = match.group(2)
            if match.group(3).find('http') == -1:
                full_url = u'http://%s' % full_url
            display_url = full_url
            if len(display_url) > 30:
                display_url = u'%s…' % display_url[0:30]
            _link = '%s<a href="%s"%s>%s</a>' % ( match.group(1), full_url, html_attrs, display_url )
            self.text = self.text.replace(match.group(0), _link)
            del(_link)
            del(full_url)
            del(display_url)

        if self.parent and hasattr(self.parent, 'text'):
            self.parent.text = self.text
        if self.parent and hasattr(self.parent, 'has_been_linked'):
            self.parent.has_been_linked = True
    
        del(matches)
        del(html_attrs)
        del(kwargs)
        del(defaults)
    
        return self.text



# encoding=utf-8

import twitter_text
from twitter_text.unicode import force_unicode

text = force_unicode('@foo said the funniest thing to ＠monkeybat and @bar http://dryan.net/xxxxx?param=true#hash #comedy #url')
tt = twitter_text.TwitterText(text)

def autolink_tests(tests, passed, failed):
    print u'Running Autolink tests'

    correct_auto_link = u'<a class="tweet-url username" href="http://twitter.com/foo" rel="nofollow">@foo</a> said the funniest thing to <a class="tweet-url username" href="http://twitter.com/monkeybat" rel="nofollow">＠monkeybat</a> and <a class="tweet-url username" href="http://twitter.com/bar" rel="nofollow">@bar</a> <a href="http://dryan.net/xxxxx?param=true#hash" rel="nofollow">http://dryan.net/xxxxx?param=t…</a> <a href="http://twitter.com/search?q=%23comedy" title="#comedy" class="tweet-url hashtag" rel="nofollow">#comedy</a> <a href="http://twitter.com/search?q=%23url" title="#url" class="tweet-url hashtag" rel="nofollow">#url</a>'
    correct_auto_link_with_hit_highlight = u'<a class="tweet-url username" href="http://twitter.com/foo" rel="nofollow">@foo</a> said the <em class="search-hit">funniest</em> thing to <a class="tweet-url username" href="http://twitter.com/monkeybat" rel="nofollow">＠monkeybat</a> and <a class="tweet-url username" href="http://twitter.com/bar" rel="nofollow">@bar</a> <a href="http://dryan.net/xxxxx?param=true#hash" rel="nofollow">http://dryan.net/xxxxx?param=t…</a> <a href="http://twitter.com/search?q=%23comedy" title="#comedy" class="tweet-url hashtag" rel="nofollow">#comedy</a> <a href="http://twitter.com/search?q=%23url" title="#url" class="tweet-url hashtag" rel="nofollow">#url</a>'
    correct_auto_link_usernames_or_lists = u'<a class="tweet-url username" href="http://twitter.com/foo" rel="nofollow">@foo</a> said the funniest thing to <a class="tweet-url username" href="http://twitter.com/monkeybat" rel="nofollow">＠monkeybat</a> and <a class="tweet-url username" href="http://twitter.com/bar" rel="nofollow">@bar</a> http://dryan.net/xxxxx?param=true#hash #comedy #url'
    correct_auto_link_hashtags = u'@foo said the funniest thing to ＠monkeybat and @bar http://dryan.net/xxxxx?param=true#hash <a href="http://twitter.com/search?q=%23comedy" title="#comedy" class="tweet-url hashtag" rel="nofollow">#comedy</a> <a href="http://twitter.com/search?q=%23url" title="#url" class="tweet-url hashtag" rel="nofollow">#url</a>'
    correct_auto_link_urls_custom = u'@foo said the funniest thing to ＠monkeybat and @bar <a href="http://dryan.net/xxxxx?param=true#hash" rel="nofollow">http://dryan.net/xxxxx?param=t…</a> #comedy #url'
    correct_auto_link_urls_custom_with_kwargs = u'@foo said the funniest thing to ＠monkeybat and @bar <a href="http://dryan.net/xxxxx?param=true#hash" class="boosh" rel="external" title="a link">http://dryan.net/xxxxx?param=t…</a> #comedy #url'

    autolink = twitter_text.Autolink(text)

    # test the overall auto_link method
    test_autolink = tt.autolink.auto_link()
    if test_autolink == correct_auto_link_with_hit_highlight:
        print u'\033[92m  Attached auto_link passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached auto_link failed:\033[0m'
        print u'    Expected: %s' % correct_auto_link_with_hit_highlight
        print u'    Returned: %s' % test_autolink
        failed +=1
    tests +=1

    test_autolink = autolink.auto_link()
    if test_autolink == correct_auto_link:
        print u'\033[92m  Stand alone auto_link passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone auto_link failed:\033[0m'
        print u'    Expected: %s' % correct_auto_link
        print u'    Returned: %s' % test_autolink
        failed +=1
    tests +=1
    
    if failed > 0: # we need to run the individual methods to determine what failed
        # test the auto_link_usernames_or_lists method
        this_tt = twitter_text.TwitterText(text)
        autolink = twitter_text.Autolink(text)
        
        test_autolink = this_tt.autolink.auto_link_usernames_or_lists()
        if test_autolink == correct_auto_link_usernames_or_lists:
            print u'\033[92m  Attached auto_link_usernames_or_lists passed\033[0m'
            passed += 1
        else:
            print u'\033[91m  Attached auto_link_usernames_or_lists failed:\033[0m'
            print u'    Expected: %s' % correct_auto_link_usernames_or_lists
            print u'    Returned: %s' % test_autolink
            failed +=1
        tests +=1

        test_autolink = autolink.auto_link_usernames_or_lists()
        if test_autolink == correct_auto_link_usernames_or_lists:
            print u'\033[92m  Stand alone auto_link_usernames_or_lists passed\033[0m'
            passed += 1
        else:
            print u'\033[91m  Stand alone auto_link_usernames_or_lists failed:\033[0m'
            print u'    Expected: %s' % correct_auto_link_usernames_or_lists
            print u'    Returned: %s' % test_autolink
            failed +=1
        tests +=1
        
        # test the auto_link_hashtags method
        this_tt = twitter_text.TwitterText(text)
        autolink = twitter_text.Autolink(text)
        
        test_autolink = this_tt.autolink.auto_link_hashtags()
        if test_autolink == correct_auto_link_hashtags:
            print u'\033[92m  Attached auto_link_hashtags passed\033[0m'
            passed += 1
        else:
            print u'\033[91m  Attached auto_link_hashtags failed:\033[0m'
            print u'    Expected: %s' % correct_auto_link_hashtags
            print u'    Returned: %s' % test_autolink
            failed +=1
        tests +=1

        test_autolink = autolink.auto_link_hashtags()
        if test_autolink == correct_auto_link_hashtags:
            print u'\033[92m  Stand alone auto_link_hashtags passed\033[0m'
            passed += 1
        else:
            print u'\033[91m  Stand alone auto_link_hashtags failed:\033[0m'
            print u'    Expected: %s' % correct_auto_link_hashtags
            print u'    Returned: %s' % test_autolink
            failed +=1
        tests +=1

        # test the auto_link_urls_custom
        this_tt = twitter_text.TwitterText(text)
        autolink = twitter_text.Autolink(text)
        
        test_autolink = this_tt.autolink.auto_link_urls_custom()
        if test_autolink == correct_auto_link_urls_custom:
            print u'\033[92m  Attached auto_link_urls_custom passed\033[0m'
            passed += 1
        else:
            print u'\033[91m  Attached auto_link_urls_custom failed:\033[0m'
            print u'    Expected: %s' % correct_auto_link_urls_custom
            print u'    Returned: %s' % test_autolink
            failed +=1
        tests +=1

        test_autolink = autolink.auto_link_urls_custom()
        if test_autolink == correct_auto_link_urls_custom:
            print u'\033[92m  Stand alone auto_link_urls_custom passed\033[0m'
            passed += 1
        else:
            print u'\033[91m  Stand alone auto_link_urls_custom failed:\033[0m'
            print u'    Expected: %s' % correct_auto_link_urls_custom
            print u'    Returned: %s' % test_autolink
            failed +=1
        tests +=1
        
    # test the auto_link_urls_custom with some kwargs for HTML attrs
    this_tt = twitter_text.TwitterText(text)
    autolink = twitter_text.Autolink(text)
    
    test_autolink = this_tt.autolink.auto_link_urls_custom(rel = 'external', class_name = 'boosh', title = 'a link')
    if test_autolink == correct_auto_link_urls_custom_with_kwargs:
        print u'\033[92m  Attached auto_link_urls_custom with kwargs passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached auto_link_urls_custom with kwargs failed:\033[0m'
        print u'    Expected: %s' % correct_auto_link_urls_custom_with_kwargs
        print u'    Returned: %s' % test_autolink
        failed +=1
    tests +=1

    test_autolink = autolink.auto_link_urls_custom(rel = 'external', class_name = 'boosh', title = 'a link')
    if test_autolink == correct_auto_link_urls_custom_with_kwargs:
        print u'\033[92m  Stand alone auto_link_urls_custom with kwargs passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone auto_link_urls_custom with kwargs failed:\033[0m'
        print u'    Expected: %s' % correct_auto_link_urls_custom_with_kwargs
        print u'    Returned: %s' % test_autolink
        failed +=1
    tests +=1


    return tests, passed, failed

def extractor_tests(tests, passed, failed):
    print u'Running Extractor tests'

    correct_mentioned_screen_names = [u'foo', u'monkeybat', u'bar']
    correct_mentioned_screen_names_with_indices = [{'indices': (0, 4), 'screen_name': u'foo'}, {'indices': (32, 42), 'screen_name': u'monkeybat'}, {'indices': (47, 51), 'screen_name': u'bar'}]
    correct_reply_screen_name = 'foo'
    correct_urls = [u'http://dryan.net/xxxxx?param=true#hash']
    correct_urls_with_indices = [{'url': u'http://dryan.net/xxxxx?param=true#hash', 'indices': (52, 90)}]
    correct_hashtags = [u'comedy', u'url']
    correct_hashtags_with_indices = [{'indices': (91, 98), 'hashtag': u'comedy'}, {'indices': (99, 103), 'hashtag': u'url'}]

    extractor = twitter_text.Extractor(text)
    
    if tt.extractor.extract_mentioned_screen_names() == correct_mentioned_screen_names:
        print u'\033[92m  Attached extract_mentioned_screen_names passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_mentioned_screen_names failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_mentioned_screen_names)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_mentioned_screen_names())
        failed +=1
    tests +=1

    if extractor.extract_mentioned_screen_names() == correct_mentioned_screen_names:
        print u'\033[92m  Stand alone extract_mentioned_screen_names passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_mentioned_screen_names failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_mentioned_screen_names)
        print u'    Returned: %s' % force_unicode(extractor.extract_mentioned_screen_names())
        failed +=1
    tests +=1

    if tt.extractor.extract_mentioned_screen_names_with_indices() == correct_mentioned_screen_names_with_indices:
        print u'\033[92m  Attached extract_mentioned_screen_names_with_indices passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_mentioned_screen_names_with_indices failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_mentioned_screen_names_with_indices)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_mentioned_screen_names_with_indices())
        failed += 1
    tests += 1

    if extractor.extract_mentioned_screen_names_with_indices() == correct_mentioned_screen_names_with_indices:
        print u'\033[92m  Stand alone extract_mentioned_screen_names_with_indices passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_mentioned_screen_names_with_indices failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_mentioned_screen_names_with_indices)
        print u'    Returned: %s' % force_unicode(extractor.extract_mentioned_screen_names_with_indices())
        failed += 1
    tests += 1

    if tt.extractor.extract_reply_screen_name() == correct_reply_screen_name:
        print u'\033[92m  Attached extract_reply_screen_name passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_reply_screen_name failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_reply_screen_name)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_reply_screen_name())
        failed +=1
    tests +=1

    if extractor.extract_reply_screen_name() == correct_reply_screen_name:
        print u'\033[92m  Stand alone extract_reply_screen_name passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_reply_screen_name failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_reply_screen_name)
        print u'    Returned: %s' % force_unicode(extractor.extract_reply_screen_name())
        failed +=1
    tests +=1

    if tt.extractor.extract_urls() == correct_urls:
        print u'\033[92m  Attached extract_urls passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_urls failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_urls)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_urls())
        failed +=1
    tests +=1

    if extractor.extract_urls() == correct_urls:
        print u'\033[92m  Stand alone extract_urls passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_urls failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_urls)
        print u'    Returned: %s' % force_unicode(extractor.extract_urls())
        failed +=1
    tests +=1

    if tt.extractor.extract_urls_with_indices() == correct_urls_with_indices:
        print u'\033[92m  Attached extract_urls_with_indices passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_urls_with_indices failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_urls_with_indices)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_urls_with_indices())
        failed += 1
    tests += 1

    if extractor.extract_urls_with_indices() == correct_urls_with_indices:
        print u'\033[92m  Stand alone extract_urls_with_indices passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_urls_with_indices failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_urls_with_indices)
        print u'    Returned: %s' % force_unicode(extractor.extract_urls_with_indices())
        failed += 1
    tests += 1

    if tt.extractor.extract_hashtags() == correct_hashtags:
        print u'\033[92m  Attached extract_hashtags passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_hashtags failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_hashtags)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_hashtags())
        failed +=1
    tests +=1

    if extractor.extract_hashtags() == correct_hashtags:
        print u'\033[92m  Stand alone extract_hashtags passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_hashtags failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_hashtags)
        print u'    Returned: %s' % force_unicode(extractor.extract_hashtags())
        failed +=1
    tests +=1

    if tt.extractor.extract_hashtags_with_indices() == correct_hashtags_with_indices:
        print u'\033[92m  Attached extract_hashtags_with_indices passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached extract_hashtags_with_indices failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_hashtags_with_indices)
        print u'    Returned: %s' % force_unicode(tt.extractor.extract_hashtags_with_indices())
        failed += 1
    tests += 1
        
    if extractor.extract_hashtags_with_indices() == correct_hashtags_with_indices:
        print u'\033[92m  Stand alone extract_hashtags_with_indices passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone extract_hashtags_with_indices failed:\033[0m'
        print u'    Expected: %s' % force_unicode(correct_hashtags_with_indices)
        print u'    Returned: %s' % force_unicode(extractor.extract_hashtags_with_indices())
        failed += 1
    tests += 1
    
    return tests, passed, failed
    
def highlighter_tests(tests, passed, failed):
    print u'Running HitHighlighter tests'
    
    correct_hit_highlight = u'@foo said the <em class="search-hit">funniest</em> thing to ＠monkeybat and @bar http://dryan.net/xxxxx?param=true#hash #comedy #url'
    
    highlighter = twitter_text.HitHighlighter(text)

    test_highlight = tt.highlighter.hit_highlight('funniest')
    if test_highlight == correct_hit_highlight:
        print u'\033[92m  Attached hit_highlight passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached hit_highlight failed:\033[0m'
        print u'    Expected: %s' % correct_hit_highlight
        print u'    Returned: %s' % test_highlight
        failed += 1
    tests += 1
        
    test_highlight = highlighter.hit_highlight('funniest')
    if test_highlight == correct_hit_highlight:
        print u'\033[92m  Stand alone hit_highlight passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone hit_highlight failed:\033[0m'
        print u'    Expected: %s' % correct_hit_highlight
        print u'    Returned: %s' % test_highlight
        failed += 1
    tests += 1
    
    return tests, passed, failed
    
def validation_tests(tests, passed, failed):
    print u'Running Validation tests'
    
    validation = twitter_text.Validation(text)
    
    if tt.validation.tweet_length() == len(text):
        print u'\033[92m  Attached tweet_length passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached tweet_length failed:\033[0m'
        print u'    Expected: %d' % len(text)
        print u'    Returned: %d' % tt.validation.tweet_length()
        failed += 1
    tests += 1

    if validation.tweet_length() == len(text):
        print u'\033[92m  Stand alone tweet_length passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone tweet_length failed:\033[0m'
        print u'    Expected: %d' % len(text)
        print u'    Returned: %d' % validation.tweet_length()
        failed += 1
    tests += 1

    if tt.validation.tweet_invalid() == (False, None):
        print u'\033[92m  Attached tweet_invalid passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Attached tweet_invalid failed:\033[0m'
        print u'    Expected: %s' % 'False, None'
        print u'    Returned: %s, %s' % tt.validation.tweet_invalid()
        failed += 1
    tests += 1

    if validation.tweet_invalid() == (False, None):
        print u'\033[92m  Stand alone tweet_invalid passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Stand alone tweet_invalid failed:\033[0m'
        print u'    Expected: %s' % 'False, None'
        print u'    Returned: %s, %s' % validation.tweet_invalid()
        failed += 1
    tests += 1
    
    print ''
    print u'Running Validation tests on bad text'
    bad_tweets = {
        'empty': u'',
        'too_long': text + text,
        'invalid_characters': text
    }
    
    this_tt = twitter_text.TwitterText(bad_tweets['empty'])
    if this_tt.validation.tweet_invalid() == (True, 'Empty text'):
        print u'\033[92m  Empty Text tweet_invalid passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Empty Text tweet_invalid failed:\033[0m'
        print u'    Expected: %s' % 'True, Empty text'
        print u'    Returned: %s, %s' % validation.tweet_invalid()
        failed += 1
    tests += 1
        
    this_tt = twitter_text.TwitterText(bad_tweets['too_long'])
    if this_tt.validation.tweet_invalid() == (True, 'Too long'):
        print u'\033[92m  Too Long tweet_invalid passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Too Long tweet_invalid failed:\033[0m'
        print u'    Expected: %s' % 'True, Too long'
        print u'    Returned: %s, %s' % validation.tweet_invalid()
        failed += 1
    tests += 1
    
    missed_bad_characters = []
    for bad_character in validation.INVALID_CHARACTERS:
        this_text = force_unicode(bad_tweets['invalid_characters'] + bad_character)
        this_tt = twitter_text.TwitterText(this_text)
        if not this_tt.validation.tweet_invalid() == (True, 'Invalid characters'):
            missed_bad_characters.append(bad_character)
    if not len(missed_bad_characters):
        print u'\033[92m  Invalid Characters tweet_invalid passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Invalid Characters tweet_invalid failed:\033[0m'
        print u'    Expected: %s' % '[]'
        print u'    Returned: %s' % force_unicode(missed_bad_characters)
        failed += 1
    tests += 1
    
    return tests, passed, failed
    
def templatetag_tests(tests, passed, failed):
    print u'Running Django templatetag tests'
    from twitter_text.templatetags import twitterize

    correct_auto_link = u'<a class="tweet-url username" href="http://twitter.com/foo" rel="nofollow">@foo</a> said the funniest thing to <a class="tweet-url username" href="http://twitter.com/monkeybat" rel="nofollow">＠monkeybat</a> and <a class="tweet-url username" href="http://twitter.com/bar" rel="nofollow">@bar</a> <a href="http://dryan.net/xxxxx?param=true#hash" rel="nofollow">http://dryan.net/xxxxx?param=t…</a> <a href="http://twitter.com/search?q=%23comedy" title="#comedy" class="tweet-url hashtag" rel="nofollow">#comedy</a> <a href="http://twitter.com/search?q=%23url" title="#url" class="tweet-url hashtag" rel="nofollow">#url</a>'
    correct_auto_link_with_hit_highlight = u'<a class="tweet-url username" href="http://twitter.com/foo" rel="nofollow">@foo</a> said the <em class="search-hit">funniest</em> thing to <a class="tweet-url username" href="http://twitter.com/monkeybat" rel="nofollow">＠monkeybat</a> and <a class="tweet-url username" href="http://twitter.com/bar" rel="nofollow">@bar</a> <a href="http://dryan.net/xxxxx?param=true#hash" rel="nofollow">http://dryan.net/xxxxx?param=t…</a> <a href="http://twitter.com/search?q=%23comedy" title="#comedy" class="tweet-url hashtag" rel="nofollow">#comedy</a> <a href="http://twitter.com/search?q=%23url" title="#url" class="tweet-url hashtag" rel="nofollow">#url</a>'
    
    if twitterize.twitter_text(text) == correct_auto_link:
        print u'\033[92m  Django templatetag with no search term passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Django templatetag with no search term failed:\033[0m'
        print u'    Expected: %s' % correct_auto_link
        print u'    Returned: %s' % twitterize.twitter_text(text)
        failed += 1
    tests += 1

    if twitterize.twitter_text(text, 'funniest') == correct_auto_link_with_hit_highlight:
        print u'\033[92m  Django templatetag with search term passed\033[0m'
        passed += 1
    else:
        print u'\033[91m  Django templatetag with search term failed:\033[0m'
        print u'    Expected: %s' % correct_auto_link_with_hit_highlight
        print u'    Returned: %s' % twitterize.twitter_text(text, 'funniest')
        failed += 1
    tests += 1
    
    return tests, passed, failed

def run_all():
    tests, passed, failed = 0, 0, 0

    tests, passed, failed = validation_tests(tests, passed, failed)

    print ''
    tests, passed, failed = extractor_tests(tests, passed, failed)

    print ''
    tests, passed, failed = highlighter_tests(tests, passed, failed)

    print ''
    tests, passed, failed = autolink_tests(tests, passed, failed)

    print ''
    tests, passed, failed = templatetag_tests(tests, passed, failed)

    print ''
    print u'Checking hit_highlight assertion that text does not have HTML tags already present'
    try:
        tt.highlighter.hit_highlight('funniest')
        print u'\033[91m  hit_highlight HTML tag check failed\033[0m'
        failed += 1
    except AssertionError:
        print u'\033[92m  hit_highlight HTML tag check passed\033[0m'
        passed += 1
    tests += 1

    results = u'%d tests run. \033[92m%d passed.\033[0m' % (tests, passed)
    if failed > 0:
        results = u'%s \033[91m%d failed.\033[0m' % (results, failed)
    print ''
    print u'\033[1m%s\033[0;0m' % results
        
if __name__ == '__main__':
    run_all()


# encoding=utf-8

from twitter_text.autolink import Autolink
from twitter_text.extractor import Extractor
from twitter_text.highlighter import HitHighlighter
from twitter_text.validation import Validation
from twitter_text.unicode import force_unicode

class TwitterText(object):
    def __init__(self, text):
        self.text = force_unicode(text) # this will get modified by some functions
        self.original_text = self.text # this never changes; use it as a fallback or for comparison
        self.has_been_linked = False
        self.tweet_length = None # gets changed by validation method
        self.tweet_is_valid = None # gets changed by validation method
        self.validation_error = None # gets changed by validation method
        
    def __unicode__(self):
        return self.text
        
    def __repr__(self):
        return self.__unicode__()
    
    @property
    def autolink(self):
        return Autolink(self.text, parent = self)
        
    @property
    def extractor(self):
        return Extractor(self.text)
        
    @property
    def highlighter(self):
        return HitHighlighter(self.text, parent = self)
        
    @property
    def validation(self):
        return Validation(self.text, parent = self)


# encoding=utf-8

import re

from twitter_text.regex import UNICODE_SPACES
from twitter_text.unicode import force_unicode

class HitHighlighter(object):

    DEFAULT_HIGHLIGHT_TAG = 'em'

    DEFAULT_HIGHLIGHT_CLASS = 'search-hit'
    
    def __init__(self, text, **kwargs):
        self.text = force_unicode(text)
        self.parent = kwargs.get('parent', False)

    def hit_highlight(self, query, **kwargs):
        """
        Add <em></em> tags around occurrences of query provided in the text except for occurrences inside of hashtags.
        
        The <em></em> tags can be overridden using the highlight_tag kwarg. For example:
        
        python> HitHighlighter('test hit here').hit_highlight('hit', highlight_tag = 'strong', highlight_class = 'search-term')
                => "test <strong class='search-term'>hit</strong> here"
        """
        defaults = {
            'highlight_tag': kwargs.get('highlight_tag', self.DEFAULT_HIGHLIGHT_TAG).lower(),
            'highlight_class': kwargs.get('highlight_class', self.DEFAULT_HIGHLIGHT_CLASS).lower(),
        }
        for key, val in defaults.items():
            if not key in kwargs:
                kwargs[key] =   val
        del(defaults)
        
        tags = ( u'<%s class="%s">' % ( kwargs.get('highlight_tag'), kwargs.get('highlight_class') ), u'</%s>' % kwargs.get('highlight_tag') )
        
        tag_search = re.compile(r'(<[^>]+>)')

        assert ( not self.parent or not getattr(self.parent, 'has_been_linked', False) ) and tag_search.match(self.text) is None, 'This text has already has HTML tags present. We can\'t highlight that reliably.' # make sure links have not already been run on this text
        del(tag_search)
        
        query_search = re.compile(ur'%s' % query, re.IGNORECASE)
        matches = query_search.finditer( self.text )
        wrapped_string = u'%s%s%s'
        UNICODE_SPACES.append(hex(20))
        len_diff = 0 # used to update the match offsets after the new wrapped text is inserted
        for match in matches:
            hashtag = False
            i = match.start() + len_diff - 1
            while i >= 0:
                if hex(ord(self.text[i])) in [hex(ord('#')), '0xff03']: # we don't want to wrap parts of hashtags as the link text will be wrong
                    hashtag = True
                    break
                elif hex(ord(self.text[i])) in UNICODE_SPACES:
                    break
                i -= 1
            if not hashtag:
                before = self.text[0:len_diff + match.start()]
                try:
                    after = self.text[len_diff + match.end():len(self.text)]
                except:
                    after = ''
                len_diff += len(before + wrapped_string % ( tags[0], match.group(0), tags[1] ) + after) - len(self.text)
                self.text = before + wrapped_string % ( tags[0], match.group(0), tags[1] ) + after

        del(query_search)
        del(matches)
        
        if self.parent and hasattr(self.parent, 'text'):
            self.parent.text = self.text
            
        del(tags)
        del(kwargs)
        
        return self.text


# encoding=utf-8

import re

from twitter_text.unicode import force_unicode

class Validation(object):
    MAX_LENGTH = 140

    INVALID_CHARACTERS = (
        u'\uFFFE', u'\uFEFF',                                   # BOM
        u'\uFFFF',                                              # Special
        u'\u202A', u'\u202B', u'\u202C', u'\u202D', u'\u202E'   # Directional change
    )

    def __init__(self, text, **kwargs):
        self.text = force_unicode(text)
        self.parent = kwargs.get('parent', False)
        
    def tweet_length(self):
        """
        Returns the length of the string as it would be displayed. This is equivilent to the length of the Unicode NFC
        (See: http://www.unicode.org/reports/tr15). This is needed in order to consistently calculate the length of a
        string no matter which actual form was transmitted. For example:
        
            U+0065  Latin Small Letter E
        +   U+0301  Combining Acute Accent
        ----------
        =   2 bytes, 2 characters, displayed as é (1 visual glyph)
        … The NFC of {U+0065, U+0301} is {U+00E9}, which is a single character and a +display_length+ of 1
        
        The string could also contain U+00E9 already, in which case the canonicalization will not change the value.
        """

        assert (not self.parent or not getattr(self.parent, 'has_been_linked', False) ), 'The validator should only be run on text before it has been modified.'

        if self.parent and hasattr(self.parent, 'tweet_length'):
            self.parent.tweet_length = len(self.text)
        return len(self.text) # thanks force_unicode for making this so simple
    
    def tweet_invalid(self):
        """
        Check the text for any reason that it may not be valid as a Tweet. This is meant as a pre-validation
        before posting to api.twitter.com. There are several server-side reasons for Tweets to fail but this pre-validation
        will allow quicker feedback.
        
        Returns false if this text is valid. Otherwise one of the following Symbols will be returned:
        
            "Too long":: if the text is too long
            "Empty text":: if the text is empty
            "Invalid characters":: if the text contains non-Unicode or any of the disallowed Unicode characters
        """

        valid = True # optimism
        validation_error = None
        invalid_characters = re.compile(ur'(%s)' % ur'|'.join(self.INVALID_CHARACTERS), re.IGNORECASE)

        if not self.tweet_length():
            valid, validation_error = False, 'Empty text'
        if self.tweet_length() > self.MAX_LENGTH:
            valid, validation_error = False, 'Too long'
        if invalid_characters.search(self.text):
            valid, validation_error = False, 'Invalid characters'
            
        if self.parent and hasattr(self.parent, 'tweet_is_valid'):
            self.parent.tweet_is_valid = valid
        if self.parent and hasattr(self.parent, 'tweet_validation_error'):
            self.parent.tweet_validation_error = validation_error
            
        return (not valid, validation_error) # swap the valid value here to confirm to the original ruby version's API


#  encoding=utf-8

# A collection of regular expressions for parsing Tweet text. The regular expression
# list is frozen at load time to ensure immutability. These reular expressions are
# used throughout the Twitter classes. Special care has been taken to make
# sure these reular expressions work with Tweets in all languages.
import re, string

REGEXEN = {} # :nodoc:

# Space is more than %20, U+3000 for example is the full-width space used with Kanji. Provide a short-hand
# to access both the list of characters and a pattern suitible for use with String#split
#  Taken from: ActiveSupport::Multibyte::Handlers::UTF8Handler::UNICODE_WHITESPACE
UNICODE_SPACES = []
for space in [9, 10, 11, 12, 13, 32, 133, 160, 5760, 6158, 8192, 8193, 8194, 8195, 8196, 8197, 8198, 8199, 8200, 8201, 8202, 8232, 8233, 8239, 8287, 12288]:
    UNICODE_SPACES.append(hex(space))
REGEXEN['spaces'] = re.compile(ur'|'.join(UNICODE_SPACES))

REGEXEN['at_signs'] = re.compile(ur'[%s]' % ur'|'.join(list(u'@＠')))
REGEXEN['extract_mentions'] = re.compile(ur'(^|[^a-zA-Z0-9_])(%s)([a-zA-Z0-9_]{1,20})(?=(.|$))' % REGEXEN['at_signs'].pattern)
REGEXEN['extract_reply'] = re.compile(ur'^(?:[%s])*%s([a-zA-Z0-9_]{1,20})' % (REGEXEN['spaces'].pattern, REGEXEN['at_signs'].pattern))

REGEXEN['list_name'] = re.compile(ur'^[a-zA-Z\u0080-\u00ff].{0,79}$')

# Latin accented characters (subtracted 0xD7 from the range, it's a confusable multiplication sign. Looks like "x")
LATIN_ACCENTS = []
for accent in [192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209, 210, 211, 212, 213, 214, 216, 217, 218, 219, 220, 221, 222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 248, 249, 250, 251, 252, 253, 254, 255]:
    LATIN_ACCENTS.append(hex(accent))
REGEXEN['latin_accents'] = re.compile(ur''.join(LATIN_ACCENTS))

# Characters considered valid in a hashtag but not at the beginning, where only a-z and 0-9 are valid.
HASHTAG_CHARACTERS = re.compile(ur'[a-z0-9_%s]' % REGEXEN['latin_accents'].pattern, re.IGNORECASE) 
REGEXEN['auto_link_hashtags'] = re.compile(ur'(^|[^0-9A-Z&\/]+)(#|＃)([0-9A-Z_]*[A-Z_]+%s*)' % HASHTAG_CHARACTERS.pattern, re.IGNORECASE)
REGEXEN['auto_link_usernames_or_lists'] = re.compile(ur'([^a-zA-Z0-9_]|^)([@＠]+)([a-zA-Z0-9_]{1,20})(\/[a-zA-Z][a-zA-Z0-9\u0080-\u00ff\-]{0,79})?')
REGEXEN['auto_link_emoticon'] = re.compile(ur'(8\-\#|8\-E|\+\-\(|\`\@|\`O|\&lt;\|:~\(|\}:o\{|:\-\[|\&gt;o\&lt;|X\-\/|\[:-\]\-I\-|\/\/\/\/Ö\\\\\\\\|\(\|:\|\/\)|∑:\*\)|\( \| \))')

# URL related hash regex collection
REGEXEN['valid_preceding_chars'] = re.compile(ur"(?:[^\/\"':!=]|^|\:)")
punct = re.escape(string.punctuation)
REGEXEN['valid_domain'] = re.compile(ur'(?:[^%s\s][\.-](?=[^%s\s])|[^%s\s]){1,}\.[a-z]{2,}(?::[0-9]+)?' % (punct, punct, punct), re.IGNORECASE)
REGEXEN['valid_url_path_chars'] = re.compile(ur'[\.\,]?[a-z0-9!\*\'\(\);:=\+\$\/%#\[\]\-_,~@\.]', re.IGNORECASE)
# Valid end-of-path chracters (so /foo. does not gobble the period).
#   1. Allow ) for Wikipedia URLs.
#   2. Allow =&# for empty URL parameters and other URL-join artifacts
REGEXEN['valid_url_path_ending_chars'] = re.compile(ur'[a-z0-9\)=#\/]', re.IGNORECASE)
REGEXEN['valid_url_query_chars'] = re.compile(ur'[a-z0-9!\*\'\(\);:&=\+\$\/%#\[\]\-_\.,~]', re.IGNORECASE)
REGEXEN['valid_url_query_ending_chars'] = re.compile(ur'[a-z0-9_&=#]', re.IGNORECASE)
REGEXEN['valid_url'] = re.compile(u'''
    (%s)
    (
        (https?:\/\/|www\.)
        (%s)
        (/%s*%s?)?
        (\?%s*%s)?
    )
    ''' % (
        REGEXEN['valid_preceding_chars'].pattern,
        REGEXEN['valid_domain'].pattern,
        REGEXEN['valid_url_path_chars'].pattern,
        REGEXEN['valid_url_path_ending_chars'].pattern,
        REGEXEN['valid_url_query_chars'].pattern,
        REGEXEN['valid_url_query_ending_chars'].pattern
    ),
re.IGNORECASE + re.X)
# groups:
# 1 - Preceding character
# 2 - URL
# 3 - Protocol or www.
# 4 - Domain and optional port number
# 5 - URL path
# 6 - Query string



from setuptools import setup, find_packages
 
setup(
    name='twitter-text-py',
    version='1.0.5',
    description='A library for auto-converting URLs, mentions, hashtags, lists, etc. in Twitter text. Also does tweet validation and search term highlighting.',
    author='Daniel Ryan',
    author_email='dryan@dryan.com',
    url='http://github.com/dryan/twitter-text-py',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    include_package_data=True,
    install_requires=['setuptools'],
    license = "BSD"
)
